#!/usr/bin/python
from gurobipy import *
from inputdata import read_dat_file
from models import *
import sys

RC_EPS = 1.0e-6
RC_CONSTANT = 1.0

datafile = sys.argv[1]
width, size, amount = read_dat_file(datafile)
print '\nColumn Generation for the cutting stock problem.'

""" --- Setup Master --- """
master = Model('Master')
masterVars = []
masterCons = []
setupMaster(size, amount, width, master, masterVars, masterCons)

""" --- Setup Pricing --- """
pricer = Model('Pricing')
pricerVars = []
setupPricing(size, amount, width, pricer, pricerVars)

iterations = 0
print 'Starting Column generation:'
while True:
    iterations += 1

    """ --- Optimize Master Problem --- """
    master.optimize()
    print 'It ' + str(iterations) + ' - RMP: ' + str(round(master.ObjVal,1)) + '  Cols: ' + str(len(masterVars))

    """ --- Get Dual Variables --- """
    duals = []
    for c in masterCons:
        duals += [c.getAttr('Pi')]

    """ --- Solve Pricing Problem --- """
    pricer.setObjective(LinExpr(duals, pricerVars))
    pricer.update()
    pricer.optimize()

    reducedCost = RC_CONSTANT - pricer.ObjVal
    if reducedCost > - RC_EPS:
        break

    """ --- Add New Variable To Master Problem --- """
    newpattern = []
    for v in pricer.getVars():
        newpattern += [v.getAttr('X')]
    newcol = Column(newpattern, masterCons)
    newvar = master.addVar(0, GRB.INFINITY, 1.0, GRB.CONTINUOUS, 'Pat_'+str(len(masterVars)+1), newcol)
    masterVars += [newvar]
    master.update()

""" --- Run a Primal Heuristic --- """
LB = round(master.ObjVal,1)
for v in master.getVars():
    v.vType = GRB.INTEGER
master.update()
master.optimize()
UB = master.ObjVal
GAP = round(((UB-LB)/LB)*100,1)
print '\nComputing Primal Solution:\nLB: ' + str(LB) + ' UB: ' + str(UB) + '  Gap: ' + str(GAP) + '%'

